using System;
using System.Collections.Generic;

namespace SqlBi.Tools.SqlBulkTool {
    static class Program {
        /// <summary>
        /// 
        /// The main entry point for the application.
        /// 
        /// Beware that [STAThread] has been removed from the declaration because
        /// this program IS multithreaded and we need it not to be set to apartment.
        /// 
        /// Known limitations, still work to be done:
        /// 
        /// - The partition function must be RIGHT
        /// 
        /// </summary>
        static void Main (string[] args) {

            if (args.Length < 1) {
                Console.WriteLine ("SQL Bulk Tool");
                Console.WriteLine ("Release to public domani by sqlbi.eu");
                Console.WriteLine ("");
                Console.WriteLine ("Usage: SqlBulkTool <ConfigurationFile>");
                Console.WriteLine ("");
            } else {
                string ConfigurationFile = args[0];

                DateTime StartTime = DateTime.Now;
                Logger.LogInfo (String.Format ("Started"));

                //
                //  Loads the configuration from the XML file
                //
                Logger.LogInfo ("Reading configuration file: " + ConfigurationFile);
                ObjectXMLSerializer o = new ObjectXMLSerializer ();
                Configuration Config = new Configuration ();
                Config = (Configuration) o.Load (Config, ConfigurationFile);

                //
                //  Store configuration information into a properly constructed DataBaseCopy object
                //
                List<DatabaseCopy> DataBaseList = new List<DatabaseCopy> ();

                foreach (Configuration.DataBaseInformation Db in Config.DataBaseInformations) {
                    if (Db.Run) {
                        Logger.LogInfo (String.Format ("Database {0} ({1} tables) will be loaded", Db.Name, Db.TableInformations.Count));
                        DatabaseCopy DataBase;
                        switch (Db.DataBaseClass) {
                            case "SqlServer":
                                DataBase = new SqlDatabaseCopy ();
                                break;
                            case "OleDb":
                                DataBase = new OleDbDatabaseCopy ();
                                break;
                            case "Navision":
                                DataBase = new NavisionDataBaseCopy ();
                                break;
                            case "Firebird":
                                DataBase = new FirebirdDatabaseCopy ();
                                break;
                            case "Db2":
                                DataBase = new Db2DatabaseCopy ();
                                break;
                            default:
                                throw new ApplicationException ("Cannot determine database engine to use");

                        }
                        DataBase.SourceConnectionString = Db.SourceConnectionString;
                        DataBase.DestConnectionString = Db.DestConnectionString;
                        DataBase.Name = Db.Name;
                        foreach (Configuration.DataBaseInformation.TableInformation TableInfo in Db.TableInformations) {
                            TableCopy newTable = new TableCopy (DataBase);
                            newTable.SourceTableName = TableInfo.SourceTableName;
                            newTable.DestTableName = TableInfo.DestTableName;
                            newTable.DestTableSchema = TableInfo.DestTableSchema;
                            newTable.UpdateMaxValue = TableInfo.UpdateMaxValue;
                            newTable.UpdateMinValue = TableInfo.UpdateMinValue;
                            newTable.TruncateBeforeLoad = TableInfo.TruncateBeforeLoad;
                            DataBase.Tables.Add (newTable);
                        }
                        DataBaseList.Add (DataBase);
                    } else {
                        Logger.LogInfo (String.Format ("Database {0} ({1} tables) will NOT be loaded", Db.Name, Db.TableInformations.Count));
                    }
                }

                //
                //  Now we can start the process 
                //
                try {
                    foreach (DatabaseCopy Db in DataBaseList) {
                        Db.PrepareLoadingThreads ();
                    }
                    ThreadManager.ProcessAll (Config.MaxNumberOfThreads);
                } catch (Exception E) {
                    Logger.LogError (E.Message);
                    throw;
                }
                TimeSpan Elapsed = DateTime.Now - StartTime;
                Logger.LogInfo (String.Format ("Full process completed in {0} ", Elapsed));
            }
        }
    }
}